# TQMLS102xA on MBLS102xA carrier board

[[_TOC_]]

## Overview

### Supported Hardware

* TQMLS102xA: module
* MBLS102xA:  board

### Versions

_Bootloader:_

* U-Boot 2017.11 + TQ Patches

_Kernel:_

* Mainline: linux 5.4.87 + Patches
* Mainline + PREEMPT-RT: linux 5.4.87 + -rt48 + Patches

### Important Notes

**Attention SAI/CAN/RGMII muxing**

The MBLS102xA can be set to support SAI1/2, CAN1/2 or RGMII1.
One of these 3 interfaces can only be active at a given time due to muxing
restrictions in RCW. The CPLD on board has to be set to connect
the required interface ports to the module via DIP switch.

* For **audio**, the u-boot must have a **SAI RCW** and CPLD must be set to SAI on
* For **CAN**, the u-boot must have a **CAN RCW** and CPLD must be set to CAN on

### Known issues

* The power supply (VBUS) of the USB-OTG port can't be disabled on the 
  MBLS102xA. The port must not be used in device mode.
* Enumeration of USB devices connected to the USB-OTG port via a USB-OTG
  cable fails
* USB-HUB <X3> can not be used in u-boot
* NFS-boot is only working via X12 (`setenv ethact eTSEC1`)
* Loading the kernel image from the TFTP server only works if the image is in
  the root directory of the TFTP server.
* When using XZ compression for Linux image, decompression takes about 1 minute.
  XZ compression is used to make the kernel image small enough for the SPI.

## Usage

### Configuration

Configure BSP for usage with TQMLS102xA / MBLS102xA by calling:

* for linux/mainline with busybox init without Qt:      `tools/config-mbls102xa`
* for linux/mainline with busybox init with Qt enabled: `tools/config-mbls102xa.qt`
* for linux/mainline with systemd init without Qt:      `tools/config-mbls102xa.sd`

### Build

Build the BSP by calling

`./p images`

The build results are located in

`./platform-MBLS102xA/images`

## Artifacts

### U-Boot images

* u-boot-with-spl-pbl-[ecc-]mmcsd-2017.11.bin	- U-Boot Image with    ECC with SPL for MMC/SD Card, RGMII muxing
* u-boot-with-spl-pbl-[can-]mmcsd-2017.11.bin	- U-Boot Image without ECC with SPL for MMC/SD Card, CAN muxing
* u-boot-with-spl-pbl-[sai-]mmcsd-2017.11.bin	- U-Boot Image without ECC with SPL for MMC/SD Card, SAI muxing
* u-boot-with-spl-pbl-mmcsd-2017.11.bin		- U-Boot Image without ECC with SPL for MMC/SD Card, RGMII muxing
* u-boot-pbl.bin.ecc.bswap			- U-Boot Image with    ECC for QSPI flash, RGMII muxing
* u-boot-pbl.bin.can.bswap			- U-Boot Image without ECC for QSPI flash, CAN muxing
* u-boot-pbl.bin.sai.bswap			- U-Boot Image without ECC for QSPI flash, SAI muxing
* u-boot-pbl.bin.bswap				- U-Boot Image without ECC for QSPI flash, RGMII muxing 

TQMLS1021A-AE

* u-boot-pbl.bin.ae.ecc.bswap			- U-Boot Image with    ECC for QSPI flash, RGMII muxing

### Linux zImages

* linuximage     - for Mainline
* linuximage-rt  - for Mainline with PREEMPT-RT

### Device tree blobs

* ls1021a-mbls1021a.dtb				- default device tree without display
* ls1021a-mbls1021a-dmb-ct21.dtb		- with parallel display (DMB S070PWS19HP-FC21)
* ls1021a-mbls1021a-dmb-ct44.dtb		- with parallel display (DMB S070SWV29HG-DC44)
* ls1021a-mbls1021a-glyn-etm0700g0edh6.dtb	- with parallel display (Glyn ETM0700G0EDH6)
* ls1021a-mbls1021a-lvds-tm070jvhg33.dtb	- with LVDS display (Tianma TM070JVHG33)
* ls1021a-mbls1021a-hdmi.dtb			- with HDMI display

### Firmware (containing Linux zImages & Device tree blobs)

* firmware.img
* firmware.tgz

### RootFS

* root.ext2            - EXT file system for SD / e-MMC
* root.tgz             - tgz for NFS mount
* root.ubifs           - ubifs image for QSPI NOR
* root.ubi             - ubi partition for QSPI NOR

__Note on QSPI root filesytem:__

	UBI partition and filesystem are designed for Flash chips and
	track erase cycles for proper wear leveling. "Updating" rootfs
	on QSPI flash in U-Boot was actually a sequence of erase/write
	of MTD partition, which resets erase counter sectors from UBI.
	This contradicts the benefits of UBI and therefore
	the script for the QSPI rootfs initialization is run once
	(for first-time image programming when flash is empty anyways)
	and then disabled.
	The desirable method of following QSPI rootfs updates is using
	UBI commands in U-Boot with script update_rootfs-qspi with
	the volume image.
	> Make sure to use root.ubifs, not root.ubi! <

## HowTo

### MBLS102xA DIP Switch settings for Boot

_Note:_

* S3 is for Boot Mode.
* X means position of DIP, - means don't care

#### SD Card

only for TQMLS102xA without eMMC

|         | S3  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  X  |  X  |  -  |  -  |
| OFF     |     |     |  -  |  -  |

#### e-MMC

eMMC is a placement option on TQMLS102xA

|         | S3  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  X  |     |  -  |  -  |
| OFF     |     |  X  |  -  |  -  |

#### QSPI

##### TQMLS102xA without eMMC

|         | S3  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |     |  X  |  -  |  -  |
| OFF     |  X  |     |  -  |  -  |

#####  TQMLS102xA with eMMC

|         | S3  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |     |     |  -  |  -  |
| OFF     |  X  |  X  |  -  |  -  |

### MBLS102xA DIP Switch settings for CPLD Function Selection

_Note:_

* S9 is for CPLD Function Selection
* The settings has to match the TQMLS102xA BSP **RCW** configuration.
* X means position of DIP, - means don't care

##### HDMI

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  X  |  X  |  -  |  -  |
| OFF     |     |     |  -  |  -  |

##### LVDS

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |     |  X  |  -  |  -  |
| OFF     |  X  |     |  -  |  -  |

##### UCC

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  X  |     |  -  |  -  |
| OFF     |     |  X  |  -  |  -  |

##### LVDS_RGBINV

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |     |     |  -  |  -  |
| OFF     |  X  |  X  |  -  |  -  |

##### EC1

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  -  |  -  |  X  |  X  |
| OFF     |  -  |  -  |     |     |

##### CAN1 / CAN2

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  -  |  -  |     |  X  |
| OFF     |  -  |  -  |  X  |     |

##### SAI1 / SAI2

|         | S9  |     |     |     |
| ------- | :-: | :-: | :-: | :-: |
| DIP     |  1  |  2  |  3  |  4  |
| ON      |  -  |  -  |  X  |     |
| OFF     |  -  |  -  |     |   X |

### MBLS102xA DIP Switch settings for CAN

_Note:_

* S2 is for CAN

##### CAN

|                       | S2   |      |
| --------------------- | :--: | :--: |
| DIP                   |  1   |  2   |
| CAN interface         | CAN0 | CAN1 |
| ON  terminated (120Ω) |      |      |
| OFF not terminated    |      |      |

### MBLS102xA DIP Switch settings for RS485

_Note:_

* S2 is for RS485

##### RS485

|                       | S2  |     |
| --------------------- | :-: | :-: |
| DIP                   |  1  |  2  |
| RS485                 | RxD | TxD |
| ON  terminated (120Ω) |     |     |
| OFF not terminated    |     |     |

### SD / e-MMC system images (bootloader, firmware partition, RootFS):

* uboot-sd.img

If U-Boot with enabled DDR ECC is required, use U-Boot Image with ECC  
If U-Boot with CAN muxing is required, use U-Boot Image with CAN  
If U-Boot with SAI muxing is required, use U-Boot Image with SAI

**For e-MMC / SD images**

Replace present U-Boot in uboot-sd.img before writing it to a card, e.g.:
```
dd if=platform-MBLS102x/images/u-boot-with-spl-pbl-ecc-mmcsd-2017.11.bin 
of=platform-MBLS102x/images/uboot-sd.img bs=4k seek=1 conv=notrunc
```

### SD / e-MMC images

#### U-Boot

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv uboot <name of u-boot image>
> run update_uboot
```

#### Device Tree

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv fdt_file <name of fdt image>
> run update_fdt
```

#### Kernel

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv kernel <name of linux image>
> run update_kernel
```

### QSPI images

#### U-Boot

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv uboot-qspi <name of u-boot image>
> run update_uboot-qspi
```

#### rootfs

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv rootfs-qspi <name of ubi image>
> run update_rootfs-qspi
```

#### Device Tree

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv fdt_file <name of fdt image>
> run update_fdt-qspi
```

#### Kernel

Perform in-system update in U-Boot via TFTP, e.g.:
```
> setenv kernel <name of linux image>
> run update_kernel-qspi
```

================================================================================
## Support Wiki

See [TQ Embedded Wiki for TQMLS102xA](https://support.tq-group.com/en/layerscape/tqmls102xa)

