# TQMa335x on MBa335x carrier board

[[_TOC_]]

## Overview

### Supported Hardware:

* TQMa335x[L]: module revisions REV.020x
* MBa335x:  board revisions REV.020x

### Versions

_Bootloader:_

* uboot-tq (Based on Mainline 2019.04)

_Kernel:_

* TI-linux-5.4.43 (Based on TI linux-5.4.y-07.00.00.005-rt)

### Known issues

* TQMa335x[L] on MBa335x: sometimes after booting the ethernet phy shows
  up wrong addresses or can not be found. This needs a hardware fix on MBa335x.
  Please contact TQ-Systems support
* TQMa335x[L] on MBa335x: U-Boot: only the first port of the CPSW is working.
  This is a driver limitation in the used U-Boot version, under Linux both ports
  are working.
* TQMa335xL: warnings for non exisiting chips when booting Linux. Disable
  RTC / EEPROM / temperature sensor in DT.

## Usage

### Configuration

Configure BSP for usage with TQMa335x[L] / MBa335x by calling:

* for linux/mainline with busybox init without Qt:      `tools/config-mba335x.tiny`

### Build

Build the BSP by calling

`./p images`

The build results are located in
`./platform-MBa335x/images`

## Artifacts

### U-Boot images

U-Boot images

* u-boot-mba335x-mmc-MLO - SPL U-Boot for SD / e-MMC boot
* u-boot-mba335x-mmc.img - U-Boot for SD / e-mmc boot

### Linux zImages

* linuximage

### Device tree blobs

* am335x-mba335x.dtb                         - without display
* am335x-mba335x-dmb-ct44.dtb                - with parallel display (DMB S070WV106H-CT44/S070SWV29HG-DC44)
* am335x-mba335x-glyn-etm0700.dtb            - with parallel display (GLYN ETM0700G0EDH6)
* am335x-mba335x-lvds-g104x1.dtb             - with LVDS display (Tianma TM070JVHG33)

### RootFS

* root.ext2                                - EXT file system for SD / e-MMC
* root.tgz                                 - tgz for NFS mount

### SD / e-MMC system images (bootloader, firmware partition, RootFS):

* image-u-boot-mba335x-mmc-hd.img		- Complete Image

## HowTo

### DIP Switches

| Switch  | Description       |
| ------- | :---------------: |
| S1      | Oscilator         |
| S2      | Boot device       |
| S3      | CAN termination   |
| S4      | RS485 Termination |

### Boot sequence

#### SD Card

Boot sequence: MMC0 (SD) → SPI0 (NOR) → UART0 (N/A) → USB0 (N/A)

| S2      |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |
| ------- | :-: | :-: | :-: | :-: | :-: | :-: | :-: | :-: |
| ON      |     |     |     |  x  |     |     |     |     |
| OFF     |  x  |  x  |  x  |     |  x  |  x  |  x  |  x  |

#### e-MMC

Boot sequence: MMC1 (e-MMC) → SPI0 (NOR) → UART0 (N/A) → USB0 (N/A)

| S2      |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |
| ------- | :-: | :-: | :-: | :-: | :-: | :-: | :-: | :-: |
| ON      |  x  |  x  |     |     |     |     |     |     |
| OFF     |     |     |  x  |  x  |  x  |  x  |  x  |  x  |

#### SPI NOR

Boot sequence: SPI0 (NOR) → MMC0 (SD) → USB0 (N/A) → UART0 (N/A)

| S2      |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |
| ------- | :-: | :-: | :-: | :-: | :-: | :-: | :-: | :-: |
| ON      |  x  |  x  |  x  |     |     |     |     |     |
| OFF     |     |     |     |  x  |  x  |  x  |  x  |  x  |

### Program complete system image

To program complete system image to SD / e-MMC, write
[SD / e-MMC system images](#sd-e-mmc-system-images-bootloader-firmware-partition-rootfs)
to SD / e-MMC at offset 0x00 / block #0

```
dd if=<SD / e-MMC system images> of=/dev/mmcblk0 bs=4k conv=fsync
```

### Update components via U-Boot

#### U-Boot environment variables

For ease of development a set of variables and scripts are in default env.
Depending on your configuration some variable values needs to bet changend
to the right values. For files to use see the [artifacts](#artifacts) section.

_Note_: Update and start scripts expect a partitioned / initialized SD-Card or
e-MMC.

* `uboot`: name of U-Boot payload image for SD / e-MMC (default = u-boot.img)
* `mlo`: name of U-Boot SPL image for SD / e-MMC (default = MLO)
* `mmcdev`: 1 for e-MMC, 2 for SD-Card (automatically generated when booting
   from SD / e-MMC with the index of the boot device, can be overwritten;
* `fdtfile`: device tree blob,
* `bootfile`: kernel image,

#### SD / e-MMC

Download bootloader from TFTP and update (make sure `mmcdev` is correctly set):

```
run update_uboot_mmc
```

================================================================================

See [TQ Embedded Wiki for TQMa335x](https://support.tq-group.com/en/arm/tqma335x)
