/*!
 * generic gpio IRQ helper
 */

#include <unistd.h> /* usleep */
#include <limits.h> /* usleep */
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <getopt.h>

#include "gpio_lib.h"

void usage(const char *progname)
{
	printf("TQ GPIO interrupt test\n");
	printf("usage:\n");
	printf("%s -g <nr> -l <count>\n", progname);
	printf("\t-g:\t gpio number\n");
	printf("\t-l:\t loop count or -1 for endless\n");
	printf("\tdefaults to single shot without loops\n");
}

int main(int argc, char **argv, char **envp)
{
	unsigned long gpio = ULONG_MAX;
	signed loops = 1;
	int ch;
	char *op;
	int ret = 0;

	while (-1 != (ch = getopt(argc, argv, "g:l:h"))) {
		// check to see if a single character or long option came through
		switch (ch) {
		case 'l':
			errno = 0;
			loops = strtol(optarg, &op, 10);
			if (errno) {
				ret = errno;
				fprintf(stderr, "%s: -l needs a number\n",
					argv[0]);
				usage(argv[0]);
				ret = EINVAL;
			}
			break;
		// short option 'g'
		case 'g':
			errno = 0;
			gpio = strtoul(optarg, &op, 10);
			if (errno) {
				ret = errno;
				fprintf(stderr, "%s: -g needs an unsigned number, err %d, gpio %lu\n",
					argv[0], ret, gpio);
				usage(argv[0]);
				ret = EINVAL;
			}
			break;
		case 'h':
			ret = 0;
			usage(argv[0]);
			break;
		default:
			fprintf(stderr, "%s: opt %c\n",
				argv[0], ch);
			usage(argv[0]);
			ret = EINVAL;
		}
	}

	if (gpio == ULONG_MAX) {
		usage(argv[0]);
		ret = EINVAL;
	}

	if (!ret) {
		if (loops > 0) {
			for(; loops; --loops) {
				ret = GPIO_GetIRQ(gpio);
				if (ret < 0)
					exit(EIO);
				printf("GPIO %lu, value is %d (%d left)\n",
					gpio, ret, (loops < 0) ? -1 : loops);
			}
		} else {
			for(; loops; ) {
				ret = GPIO_GetIRQ(gpio);
				if (ret < 0)
					exit(EIO);
				printf("GPIO %lu, value is %d (%d left)\n",
					gpio, ret, (loops < 0) ? -1 : loops);
			}
		}
	}

	return ret;
}
