/*!
 * generic gpio IRQ helper
 */

#include <unistd.h> /* usleep */
#include <limits.h> /* usleep */
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <getopt.h>

#include "gpio_lib.h"

void usage(const char *progname)
{
	printf("TQ GPIO test\n");
	printf("usage:\n");
	printf("%s -g <nr> -o <operation>\n", progname);
	printf("\t-g:\t gpio number\n");
	printf("\t-o:\t <export, unexport, input, output, high, low, falling, rising, read>\n");
}

int main(int argc, char **argv, char **envp)
{
	unsigned long gpio = ULONG_MAX;
	int ch;
	char *op;
	int ret = 0;

	while (-1 != (ch = getopt(argc, argv, "g:o:h"))) {
		// check to see if a single character or long option came through
		switch (ch) {
		case 'o':
			errno = 0;
			op = optarg;
			break;
		// short option 'g'
		case 'g':
			errno = 0;
			gpio = strtoul(optarg, &op, 10);
			if (errno) {
				ret = errno;
				fprintf(stderr, "%s: -g needs an unsigned number, err %d, gpio %lu\n",
					argv[0], ret, gpio);
				usage(argv[0]);
				ret = EINVAL;
			}
			break;
		case 'h':
			ret = 0;
			usage(argv[0]);
			break;
		default:
			fprintf(stderr, "%s: opt %c\n",
				argv[0], ch);
			usage(argv[0]);
			ret = EINVAL;
		}
	}

	if (gpio == ULONG_MAX) {
		usage(argv[0]);
		ret = EINVAL;
	}

	if (!ret) {
		if (0 == strcmp(op, "export")) {
			ret = GPIO_Register(gpio);
		} else if (0 == strcmp(op, "unexport")) {
			ret = GPIO_Free(gpio);
		} else if (0 == strcmp(op, "input")) {
			ret = GPIO_SetDir(gpio, 0);
		} else if (0 == strcmp(op, "output")) {
			ret = GPIO_SetDir(gpio, 1);
		} else if (0 == strcmp(op, "read")) {
			ret = GPIO_GetVal(gpio);
			if (ret >= 0) {
				fprintf(stdout, "%d\n", ret);
			}
		} else if (0 == strcmp(op, "high")) {
			ret = GPIO_SetVal(gpio, 1);
		} else if (0 == strcmp(op, "low")) {
			ret = GPIO_SetVal(gpio, 0);
		} else if (0 == strcmp(op, "toggle")) {
			GPIO_Toggle(gpio);
			ret = 0;
		} else if (0 == strcmp(op, "falling")) {
			ret = GPIO_SetEdge(gpio, "falling");
		} else if (0 == strcmp(op, "rising")) {
			ret = GPIO_SetEdge(gpio, "rising");
		} else if (0 == strcmp(op, "both")) {
			ret = GPIO_SetEdge(gpio, "both");
		} else {
			ret = EINVAL;
			fprintf(stderr, "%s: operation %s not supported\n",
				argv[0], op);
		}
	}

	return ret;
}
