/* porttest -- put a USB hub port into TEST mode */
/* To build:  gcc -o testport testport.c */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>

#include <linux/usbdevice_fs.h>
#include <linux/usb/ch9.h>

#define	USB_MAXCHILDREN		31

#include <linux/usb/ch11.h>

char *mode_names[] = {
	"Reserved",		/* 0 */
	"Test_J",		/* 1 */
	"Test_K",		/* 2 */
	"Test_SE0_NAK",		/* 3 */
	"Test_Packet",		/* 4 */
	"Test_Force_Enable",	/* 5 */
	/* Remaining values are reserved */
};
#define MAX_TEST_MODE		5

int main(int argc, char **argv)
{
	const char *filename;
	int portnum, testmode;
	int fd;
	int rc;
	struct usbdevfs_ctrltransfer ctl;

	if (argc != 4) {
		fprintf(stderr, "Usage: porttest device-filename portnum testmode\n");
		return 1;
	}
	filename = argv[1];

	errno = 0;
	portnum = strtol(argv[2], 0, 10);
	if (errno || portnum <= 0 || portnum > USB_MAXCHILDREN) {
		fprintf(stderr, "Invalid port number: %d\n", portnum);
		return 1;
	}

	errno = 0;
	testmode = strtol(argv[3], 0, 10);
	if (errno || testmode <= 0 || testmode > MAX_TEST_MODE) {
		fprintf(stderr, "Invalid test mode: %d\n", testmode);
		return 1;
	}

	fd = open(filename, O_WRONLY);
	if (fd < 0) {
		perror("Error opening device file");
		return 1;
	}

	printf("Setting port %d to test mode %d (%s)\n", portnum, testmode,
			mode_names[testmode]);

	ctl.bRequestType = USB_DIR_OUT | USB_RT_PORT;
	ctl.bRequest = USB_REQ_SET_FEATURE;
	ctl.wValue = USB_PORT_FEAT_TEST;
	ctl.wIndex = (testmode << 8) | portnum;
	ctl.wLength = 0;

	rc = ioctl(fd, USBDEVFS_CONTROL, &ctl);
	if (rc < 0) {
		perror("Error in ioctl");
		return 1;
	}
	printf("Test mode successful\n");

	close(fd);
	return 0;
}
