# -*-makefile-*-
#
# Copyright (C) 2015-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel, Max Merchel
#
# See CREDITS for details about who has contributed to this project.
#
# For further information about the PTXdist project and license conditions
# see the README file.
#

#
# We provide this package
#
PACKAGES-$(PTXCONF_KERNEL_RT) += kernel-rt

#
# Paths and names
#
KERNEL_RT_VERSION	:= $(call remove_quotes,$(PTXCONF_KERNEL_RT_VERSION))
KERNEL_RT_MD5		:= $(call remove_quotes,$(PTXCONF_KERNEL_RT_MD5))
KERNEL_RT		:= linux-rt-$(KERNEL_RT_VERSION)
ifneq ($(KERNEL_RT_NEEDS_GIT_URL),y)
KERNEL_RT_SUFFIX		:= tar.xz
else
KERNEL_RT_SUFFIX		:= tar.gz
endif
KERNEL_RT_DIR		:= $(BUILDDIR)/linux-rt-$(KERNEL_RT_VERSION)
KERNEL_RT_CONFIG	:= $(call ptx/in-platformconfigdir, $(call remove_quotes, $(PTXCONF_KERNEL_RT_CONFIG)))
KERNEL_RT_LICENSE	:= GPL-2.0-only
ifdef PTXCONF_KERNEL_RT_SOURCE_KERNEL_ORG
ifneq ($(KERNEL_RT_NEEDS_GIT_URL),y)
KERNEL_RT_URL		:= $(call kernel-url, KERNEL_RT)
else
KERNEL_RT_URL		:= https://git.kernel.org/pub/scm/linux/kernel/git/rt/linux-stable-rt.git/snapshot/$(KERNEL_RT).$(KERNEL_RT_SUFFIX)
endif
endif
ifdef PTXCONF_KERNEL_SOURCE_TQ
KERNEL_RT_URL		:= https://github.com/tq-systems/linux-tqmaxx.git;tag=$(KERNEL_RT_VERSION)
endif

KERNEL_RT_SOURCE	:= $(SRCDIR)/$(KERNEL_RT).$(KERNEL_RT_SUFFIX)
KERNEL_RT_DEVPKG	:= NO

KERNEL_RT_PKGDIR	:= $(PKGDIR)/$(KERNEL_RT)
KERNEL_RT_KERNELIMAGE	:= $(IMAGEDIR)/linuximage-rt

# ----------------------------------------------------------------------------
# Prepare
# ----------------------------------------------------------------------------

KERNEL_RT_IMG_DIR	:= ${KERNEL_RT_DIR}/arch/${PTXCONF_KERNEL_RT_ARCH_STRING}/boot
KERNEL_RT_DTS_DIR	:= ${KERNEL_RT_DIR}/arch/${PTXCONF_KERNEL_RT_ARCH_STRING}/boot/dts

# use CONFIG_CC_STACKPROTECTOR if available. The rest makes no sense for the kernel
KERNEL_RT_WRAPPER_BLACKLIST := \
	TARGET_HARDEN_STACK \
	TARGET_HARDEN_FORTIFY \
	TARGET_HARDEN_RELRO \
	TARGET_HARDEN_BINDNOW \
	TARGET_HARDEN_PIE \
	TARGET_DEBUG \
	TARGET_BUILD_ID

KERNEL_RT_PATH		:= PATH=$(CROSS_PATH)
KERNEL_RT_ENV		:= \
	KCONFIG_NOTIMESTAMP=1 \
	pkg_wrapper_blacklist="$(KERNEL_RT_WRAPPER_BLACKLIST)"

# check for old kernel modules rules
KERNEL_RT_MAKEVARS = -C KERNEL_RT_MAKEVARS-was-renamed-to-KERNEL_RT_MAKE_OPT
$(STATEDIR)/kernel-rt.% kernel-rt_%config $(IMAGE_KERNEL_RT_IMAGE) $(KERNEL_RT_SOURCE): KERNEL_RT_MAKEVARS=

KERNEL_RT_CONF_OPT := \
	V=$(PTXDIST_VERBOSE) \
	ARCH=$(PTXCONF_KERNEL_RT_ARCH_STRING) \
	CROSS_COMPILE=$(KERNEL_CROSS_COMPILE) \
	INSTALL_MOD_PATH=$(KERNEL_RT_PKGDIR) \
	PTX_KERNEL_DIR=$(KERNEL_RT_DIR)

ifdef PTXCONF_KERNEL_RT_MODULES_INSTALL
KERNEL_RT_CONF_OPT += \
	DEPMOD=$(PTXCONF_SYSROOT_HOST)/sbin/depmod
endif

ifndef PTXCONF_KERNEL_RT_GCC_PLUGINS
KERNEL_RT_CONF_OPT += \
	HOSTCXX=false
endif

#
# support the different kernel image formats
#
KERNEL_RT_IMAGE := $(call remove_quotes, $(PTXCONF_KERNEL_RT_IMAGE))

# these are sane default
KERNEL_RT_IMAGE_PATH_y := $(KERNEL_RT_DIR)/arch/$(PTXCONF_KERNEL_RT_ARCH_STRING)/boot/$(KERNEL_RT_IMAGE)

KERNEL_RT_DTBS = $(call remove_quotes,$(PTXCONF_KERNEL_RT_DTS))
KERNEL_RT_DTBS := $(notdir $(KERNEL_RT_DTBS))
KERNEL_RT_DTBS := $(basename $(KERNEL_RT_DTBS))
KERNEL_RT_SRC_DTBS := $(addsuffix .dtb,$(KERNEL_RT_DTBS))
KERNEL_RT_SRC_DTBS := $(addprefix $(KERNEL_RT_DTS_DIR)/,$(KERNEL_RT_SRC_DTBS))

#
# we need to apply a different suffix than .dtb to the files,
# since dtc.make contains a generic rule for dtb files referencing
# settings and vars from kernel.make
# 
KERNEL_RT_IMG_DTBS := $(addprefix $(IMAGEDIR)/,$(KERNEL_RT_DTBS))
KERNEL_RT_IMG_DTBS := $(addsuffix .dtb, $(KERNEL_RT_IMG_DTBS))
KERNEL_RT_IMG_DTBS := $(addsuffix .rt-$(KERNEL_RT_VERSION), $(KERNEL_RT_IMG_DTBS))

# don't know why it works, but we need to copy this from global rule space
# to prevent clash with normal kernel env in ptxd_make_world_init_compat
$(STATEDIR)/kernel-rt.get:
	@$(call targetinfo)
	@$(foreach src,$($(PTX_MAP_TO_PACKAGE_$(*))_SOURCES), \
		$(call world/get, $($(src)))$(ptx/nl))
	@$(foreach src,$($(PTX_MAP_TO_PACKAGE_$(*))_SOURCES), \
		$(call world/check_src, $($(src)))$(ptx/nl))
	@$(call touch)

ifdef PTXCONF_KERNEL_RT
$(KERNEL_RT_CONFIG):
	@echo
	@echo "*************************************************************************"
	@echo " Please generate a kernelconfig with 'ptxdist menuconfig kernel-rt'"
	@echo "*************************************************************************"
	@echo
	@echo
	@exit 1
endif


$(STATEDIR)/kernel-rt.prepare:
	@$(call targetinfo)

	@$(call world/kconfig-setup, KERNEL_RT)

	@$(call ptx/oldconfig, KERNEL_RT)
	@$(call world/kconfig-sync, KERNEL_RT)

	@$(call touch)


# ----------------------------------------------------------------------------
# tags
# ----------------------------------------------------------------------------

$(STATEDIR)/kernel-rt.tags:
	@$(call targetinfo)
	@$(MAKE) -C $(KERNEL_RT_DIR) $(KERNEL_RT_CONF_OPT) tags TAGS cscope

# ----------------------------------------------------------------------------
# Compile
# ----------------------------------------------------------------------------

KERNEL_RT_MAKE_OPT		:= $(KERNEL_RT_CONF_OPT)

$(STATEDIR)/kernel-rt.compile:
	@$(call targetinfo)
	@rm -f \
		$(KERNEL_RT_DIR)/usr/initramfs_data.cpio.* \
		$(KERNEL_RT_DIR)/usr/.initramfs_data.cpio.*
	@$(call compile, KERNEL_RT, $(KERNEL_RT_MAKE_OPT) $(KERNEL_RT_IMAGE) $(PTXCONF_KERNEL_RT_MODULES_BUILD) dtbs)

	@$(call touch)

# ----------------------------------------------------------------------------
# Install
# ----------------------------------------------------------------------------

KERNEL_RT_INSTALL_OPT := $(KERNEL_RT_MAKE_OPT) modules_install

$(STATEDIR)/kernel-rt.install:
	@$(call targetinfo)

ifdef PTXCONF_KERNEL_RT_MODULES_INSTALL
	@$(call world/install, KERNEL_RT)
endif

	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install
# ----------------------------------------------------------------------------

$(STATEDIR)/kernel-rt.targetinstall:
	@$(call targetinfo)

ifdef PTXCONF_KERNEL_RT_XPKG
	@$(call install_init,  kernel-rt)
	@$(call install_fixup, kernel-rt, PRIORITY,optional)
	@$(call install_fixup, kernel-rt, SECTION,base)
	@$(call install_fixup, kernel-rt, AUTHOR,"Markus Niebel, TQ-Systems GmbH")
	@$(call install_fixup, kernel-rt, DESCRIPTION,missing)


ifdef PTXCONF_KERNEL_RT_INSTALL
	@$(call install_copy, kernel-rt, 0, 0, 0644, $(KERNEL_RT_IMAGE_PATH_y), /boot/$(KERNEL_RT_IMAGE)-rt-$(KERNEL_RT_VERSION), n)
	@$(foreach image, $(KERNEL_RT_SRC_DTBS), \
		$(call install_copy, kernel-rt, 0, 0, 0644, $(image), /boot/$(notdir $(basename $(image))).dtb.rt-$(KERNEL_RT_VERSION));)
	@echo "done."
endif

	@$(call install_finish, kernel-rt)
endif
	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install-post
# ----------------------------------------------------------------------------
ifdef PTXCONF_IMAGE_KERNEL_RT_INSTALL_EARLY
$(STATEDIR)/kernel-rt.targetinstall.post: $(IMAGEDIR)/linuximage-rt
endif

$(STATEDIR)/kernel-rt.targetinstall.post:
	@$(call targetinfo)

	@echo "Creating '$(notdir $(KERNEL_RT_IMAGE))' from '$(notdir $(KERNEL_RT_IMAGE_PATH_y))'..."
	@install -m 644 "$(KERNEL_RT_IMAGE_PATH_y)" "$(KERNEL_RT_KERNELIMAGE)"

	@echo -n "Creating '$(notdir $(KERNEL_RT_IMG_DTBS))' from '$(notdir $(KERNEL_RT_SRC_DTBS))'..."
	@$(foreach image, $(KERNEL_RT_SRC_DTBS), \
		install -m 644 $(image)  $(IMAGEDIR)/$(notdir $(basename $(image))).dtb.rt-$(KERNEL_RT_VERSION);)
	@echo "done."

ifdef PTXCONF_KERNEL_RT_MODULES_INSTALL
	@$(call install_init,  kernel-rt)
	@$(call install_fixup, kernel-rt, PRIORITY,optional)
	@$(call install_fixup, kernel-rt, SECTION,base)
	@$(call install_fixup, kernel-rt, AUTHOR,"Robert Schwebel <r.schwebel@pengutronix.de>")
	@$(call install_fixup, kernel-rt, DESCRIPTION,missing)

	@$(call install_glob, kernel-rt, 0, 0, -, /lib/modules, *.ko,, k)
	@$(call install_glob, kernel-rt, 0, 0, -, /lib/modules,, *.ko */build */source, n)

	@$(call install_finish, kernel-rt)
endif

	@$(call touch)


	@$(call touch)


# ----------------------------------------------------------------------------
# Clean
# ----------------------------------------------------------------------------

$(STATEDIR)/kernel-rt.clean:
	@$(call targetinfo)
	@$(call clean_pkg, KERNEL_RT)
	@if [ -L $(KERNEL_RT_DIR) ]; then \
		$(MAKE) -C $(KERNEL_RT_DIR) $(KERNEL_RT_MAKE_OPT) distclean; \
	fi

# ----------------------------------------------------------------------------
# oldconfig / menuconfig
# ----------------------------------------------------------------------------

kernel-rt_oldconfig kernel-rt_menuconfig  kernel-rt_nconfig: $(STATEDIR)/kernel-rt.extract
	@$(call world/kconfig, KERNEL_RT, $(subst kernel-rt_,,$@))

# vim: syntax=make
