# -*-makefile-*-
#
# Copyright (C) 2013-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# See CREDITS for details about who has contributed to this project.
#
# For further information about the PTXdist project and license conditions
# see the README file.
#

#
# We provide this package
#
PACKAGES-$(PTXCONF_LINUX_FIRMWARE) += linux-firmware

LINUX_FIRMWARE_VERSION	:= 6f89735800fe
LINUX_FIRMWARE		:= linux-firmware-$(LINUX_FIRMWARE_VERSION)
LINUX_FIRMWARE_SUFFIX	:= tar.gz
LINUX_FIRMWARE_MD5	:= 45af22f615bcfc027850b4a1b81f3513

# TODO: this package uses an own get stage since the generic git fetcher will stumble
# across some repo issues for linux-firmware.git
LINUX_FIRMWARE_URL		:= \
	https://git.kernel.org/pub/scm/linux/kernel/git/firmware/linux-firmware.git

LINUX_FIRMWARE_DIR	= $(BUILDDIR)/linux-firmware-$(LINUX_FIRMWARE_VERSION)

LINUX_FIRMWARE_SOURCE	:= $(SRCDIR)/$(LINUX_FIRMWARE).$(LINUX_FIRMWARE_SUFFIX)

LINUX_FIRMWARE_LICENSE		:= proprietary
LINUX_FIRMWARE_LICENSE_FILES	:= \
	file://WHENCE;md5=f43f89689b5909a5646a2823cd6c1236

ifdef PTXCONF_LINUX_FIRMWARE_REALTEK_NIC

endif

ifdef PTXCONF_LINUX_FIRMWARE_RALINK
LINUX_FIRMWARE_LICENSE_FILES += \
	file://LICENCE.ralink-firmware.txt;md5=ab2c269277c45476fb449673911a2dfd \
	file://LICENCE.ralink_a_mediatek_company_firmware;md5=728f1a85fd53fd67fa8d7afb080bc435
endif

ifdef PTXCONF_LINUX_FIRMWARE_REDPINE_RS91X

endif

$(LINUX_FIRMWARE_SOURCE):
	@$(call targetinfo)
	cd $(SRCDIR) && \
	if [ -d git.kernel.org.pub.scm.linux.kernel.git.firmware.linux-firmware.git ]; then \
		cd git.kernel.org.pub.scm.linux.kernel.git.firmware.linux-firmware.git; \
		git fetch --prune; \
	else \
		git clone --bare $(LINUX_FIRMWARE_URL) git.kernel.org.pub.scm.linux.kernel.git.firmware.linux-firmware.git; \
		cd git.kernel.org.pub.scm.linux.kernel.git.firmware.linux-firmware.git; \
	fi && \
	git archive \
		--format=$(LINUX_FIRMWARE_SUFFIX) \
		--prefix=$(LINUX_FIRMWARE)/ \
		--output=$(SRCDIR)/$(LINUX_FIRMWARE).$(LINUX_FIRMWARE_SUFFIX) \
		$(LINUX_FIRMWARE_VERSION)

$(STATEDIR)/linux-firmware.prepare:
	@$(call targetinfo)
	@$(call touch)

$(STATEDIR)/linux-firmware.compile:
	@$(call targetinfo)
	@$(call touch)

# ----------------------------------------------------------------------------
# Install
# ----------------------------------------------------------------------------

$(STATEDIR)/linux-firmware.install:
	@$(call targetinfo)
	-rm -f $(LINUX_FIRMWARE_PKGDIR)
	mkdir -p $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
	cp $(LINUX_FIRMWARE_DIR)/WHENCE $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
ifdef PTXCONF_LINUX_FIRMWARE_RALINK
	cp $(LINUX_FIRMWARE_DIR)/LICENCE.ralink* $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
	cp $(LINUX_FIRMWARE_DIR)/rt2*.bin $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
	cp $(LINUX_FIRMWARE_DIR)/rt3*.bin $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
	cp $(LINUX_FIRMWARE_DIR)/rt73.bin $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
endif
ifdef PTXCONF_LINUX_FIRMWARE_REALTEK_NIC
	cp -r $(LINUX_FIRMWARE_DIR)/rtl_nic $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
endif
ifdef PTXCONF_LINUX_FIRMWARE_REDPINE_RS91X
	cp -r $(LINUX_FIRMWARE_DIR)/rsi $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
	cp $(LINUX_FIRMWARE_DIR)/rsi_91x.fw $(LINUX_FIRMWARE_PKGDIR)/lib/firmware
endif
ifdef PTXCONF_LINUX_FIRMWARE_IMX_SDMA
	mkdir -p $(LINUX_FIRMWARE_PKGDIR)/lib/firmware/imx
	cp -r $(LINUX_FIRMWARE_DIR)/imx/sdma $(LINUX_FIRMWARE_PKGDIR)/lib/firmware/imx
endif

	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install
# ----------------------------------------------------------------------------

$(STATEDIR)/linux-firmware.targetinstall:
	@$(call targetinfo)
	@$(call install_init, linux-firmware)
	@$(call install_fixup,linux-firmware,PRIORITY,optional)
	@$(call install_fixup,linux-firmware,SECTION,base)
	@$(call install_fixup,linux-firmware,AUTHOR,"Markus Niebel, TQ-Systems GmbH")
	@$(call install_fixup,linux-firmware,DESCRIPTION,missing)

	@$(call install_tree, linux-firmware, 0, 0, -, /lib/firmware)

	@$(call install_finish,linux-firmware)

	@$(call touch)

# vim: syntax=make
