#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        collect_hashes.sh
#
# Copyright (C) 2016-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script to collect hashes for files in a directory.
#
#              Use release.sh to perform a complete build and create archives of
#              images, source tarballs, dev packages and BSP
#
###############################################################################

# DEBUGGING
# set -e 
set -C # noclobber

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM"
    echo "    print md5sum for all files in current dir"
    echo "$PROGRAM --directory <dir>"
    echo "    printout md5sums for all files in <dir>."
    echo
}


function main () {
	MYPWD=$(pwd)
	DIRECTORY=$(pwd)

	# Process command-line arguments.
	while test $# -gt 0; do
	    case ${1} in

		--directory )
		    shift
		    DIRECTORY="${1}"
		    ;;

		-? | --usage | --help )
		    usage
		    exit 0
		    ;;

		-* )
		    echo "Unrecognized option: $1" >&2
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	if [ -d ${DIRECTORY} ]; then
		cd ${DIRECTORY}
		FILES=$(find . -maxdepth 1 -type f)

		for file in ${FILES}; do
			md5sum ${file}
			if [ "$?" -ne "0" ]; then
				echo "ERROR: ${file}"
			fi
		done
	fi
	cd ${MYPWD}
}

main $@
