#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        fsl-eula.sh
#
# Copyright (C) 2016-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: utility script handling Freescale BSP packages
#

# DEBUGGING
##set -e 
set -C # noclobber
## set -x

readonly FSL_PATTERN=EOEULA

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM accept <source> <name>-<version> <builddir>"
    echo "    query for license acceptance"
    echo "$PROGRAM extract <source> <name>-<version> <builddir>"
    echo "    extract package to build dir and extract EULA"
    echo
}

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

function accept_fsl_eula() {
	local PKG_SRC="${1}"
	local PKG_SIG="${2}"
	local WRK_DIR="${3}"

	cd ${WRK_DIR}
	pwd > /dev/null
	rm -rf ${WRK_DIR}/${PKG_SIG}
	sh ${PKG_SRC}
	result=$?
	rm -rf ${WRK_DIR}/${PKG_SIG}
	if [ "${result}" -ne "0" ]
	then
		rm -f ${PKG_SRC}
	fi
	return ${result}
}

function extract_fsl_pkg() {
	local PKG_SRC="${1}"
	local PKG_SIG="${2}"
	local WRK_DIR="${3}"

	cd ${WRK_DIR}
	pwd > /dev/null
	rm -rf ${WRK_DIR}/${PKG_SIG}
	${PKG_SRC} --force --auto-accept
	echo "calling sed  -e 1,/${FSL_PATTERN}/ d ${PKG_SRC} | sed -e '/${FSL_PATTERN}/,$ d'"
	sed -e "1,/${FSL_PATTERN}/ d" ${PKG_SRC} | sed -e "/${FSL_PATTERN}/,$ d" > ${WRK_DIR}/${PKG_SIG}/EULA

	return 0
}


function main() {
	# Process command-line arguments.
	local PKG_SRC="${2}"
	local PKG_SIG="${3}"
	local WRK_DIR="${4}"

	chmod +x ${PKG_SRC}

	case $1 in
	extract )
		extract_fsl_pkg ${PKG_SRC} ${PKG_SIG} ${WRK_DIR}
		;;
	accept )
		accept_fsl_eula ${PKG_SRC} ${PKG_SIG} ${WRK_DIR}
		;;
	-? | --usage | --help )
		usage
		exit
		;;
	esac
}

main $@
